Package: grpnet
Type: Package
Title: Group Elastic Net Regularized GLMs and GAMs
Version: 1.2
Date: 2026-04-29
Authors@R: person(c("Nathaniel", "E."), "Helwig", email = "helwig@umn.edu", role = c("aut", "cre"))
Description: Efficient algorithms for fitting generalized linear and additive models with group elastic net penalties as described in Helwig (2025) <doi:10.1080/10618600.2024.2362232>. Implements group LASSO, group MCP, and group SCAD with an optional group ridge penalty. Computes the regularization path for linear regression (gaussian), multivariate regression (multigaussian), smoothed support vector machines (svm1), squared support vector machines (svm2), logistic regression (binomial), proportional odds logistic regression (ordinal), multinomial logistic regression (multinomial), log-linear count regression (poisson and negative.binomial), and log-linear continuous regression (gamma and inverse gaussian). Supports default and formula methods for model specification, k-fold cross-validation for tuning the regularization parameters, and nonparametric regression via tensor product reproducing kernel (smoothing spline) basis function expansion. 
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 3.5.0)
NeedsCompilation: yes
Packaged: 2026-04-29 11:07:15 UTC; nate
Author: Nathaniel E. Helwig [aut, cre]
Maintainer: Nathaniel E. Helwig <helwig@umn.edu>
Repository: CRAN
Date/Publication: 2026-04-29 12:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-29 15:33:37 UTC; unix
Archs: grpnet.so.dSYM
