\name{StartupMessage}
\alias{StartupMessage}
\alias{grpnetStartupMessage}

\title{Startup Message for grpnet}

\description{
Prints the startup message when grpnet is loaded. Not intended to be called by the user.
}

\details{
The `grpnet' ascii start-up message was created using the taag software.
}

\references{
https://patorjk.com/software/taag/
}